@@ -108,7 +108,7 @@ class SLE(Equilibrium, phases='ls'):
     >>> sle
     SLE(imol=MolarFlowIndexer(phases=('l', 's'),
             l=[('Tetradecanol', 30)]),
-        thermal_condition=ThermalCondition(T=320.00, P=101325))      
+        thermal_condition=ThermalCondition(T=320.00, P=101325))
     >>> sle('Tetradecanol', H=0.) # Under melting point given H
     >>> sle
     SLE(imol=MolarFlowIndexer(phases=('l', 's'),
@@ -118,7 +118,7 @@ class SLE(Equilibrium, phases='ls'):
     >>> sle
     SLE(imol=MolarFlowIndexer(phases=('l', 's'),
             l=[('Tetradecanol', 30)]),
-        thermal_condition=ThermalCondition(T=317.59, P=101325))  
+        thermal_condition=ThermalCondition(T=317.59, P=101325))
     >>> sle('Tetradecanol', H=500000) # At melting point given H
     >>> sle
     SLE(imol=MolarFlowIndexer(
@@ -143,12 +143,8 @@ class SLE(Equilibrium, phases='ls'):
     )
     
     def __init__(self, imol=None, thermal_condition=None, thermo=None, 
-                 solubility_weight=None, activity_coefficient=None):
+                 activity_coefficient=None):
         super().__init__(imol, thermal_condition, thermo)
-        imol = self._imol
-        self._phase_data = tuple(imol)
-        self._liquid_mol = imol['l']
-        self._solid_mol = imol['s']
         self._phase_data = tuple(imol)
         self._nonzero = None
         self._index = ()
@@ -158,8 +154,10 @@ class SLE(Equilibrium, phases='ls'):
     
     def _setup(self):
         # Get flow rates
-        liquid_mol = self._liquid_mol
-        solid_mol = self._solid_mol
+        imol = self._imol
+        self._phase_data = tuple(imol)
+        self._liquid_mol = liquid_mol = imol['l']
+        self._solid_mol = solid_mol = imol['s']
         mol = liquid_mol + solid_mol
         solute_index = self._solute_index
         self._mol_solute = mol_solute = mol[solute_index]
@@ -329,4 +327,4 @@ class SLE(Equilibrium, phases='ls'):
         
         
 class SLECache(Cache): load = SLE
-del Cache, Equilibrium     
\ No newline at end of file
+del Cache, Equilibrium     
