@@ -10,27 +10,33 @@
 from . import pickle
 from . import representation
 from . import decorators
-from . import other
+from . import misc
 from . import cache
 from . import registry
 from . import colors
 from . import plots
+from . import stream_filters
+from . import abstract_method
 
 __all__ = (*pickle.__all__,
            *representation.__all__,
            *decorators.__all__,
-           *other.__all__,
+           *misc.__all__,
            *cache.__all__,
            *registry.__all__,
            *colors.__all__,
            *plots.__all__,
+           *stream_filters.__all__,
+           *abstract_method.__all__,
 )
 
 from .pickle import *
 from .representation import *
 from .decorators import *
-from .other import *
+from .misc import *
 from .cache import *
 from .registry import *
 from .colors import *
-from .plots import *
\ No newline at end of file
+from .plots import *
+from .stream_filters import *
+from .abstract_method import *
\ No newline at end of file
